import processing.pdf.*;

// Saeed Mahani
// MAT 259 - Data Visualization
// W13

import de.bezier.data.sql.*;

MySQL msql;
int categoryCount = 8; 
int yearCount = 2;
int[][] checkouts = new int[yearCount][categoryCount];
String[] categories = {"Poetry", "Mammals", "Textile Arts", "Folklore", "Health", "Diseases", "Sports", "North America", "Computer Programming"};

float maxCheckoutCount = 0;
float sqrt2 = sqrt(2.0);

String DL = ","; //the DELIMITER
PFont mainFont;
PFont categoryFont;
PFont titleFont;
PFont yearFont;

PShape s;

void setup() {
  smooth();
  
  String q = "select\n  year(activity.o) as \'Year\',\n  sum(case when (dewey.dewey = 811) then 1 else 0 end) as \'811 Poetry\',\n  sum(case when (dewey.dewey = 599) then 1 else 0 end) as \'599 Food & Drink\',\n  sum(case when (dewey.dewey = 746) then 1 else 0 end) as \'746 Textile Arts\',\n  sum(case when (dewey.dewey = 398) then 1 else 0 end) as \'398 Folklore\',\n  sum(case when (dewey.dewey = 613) then 1 else 0 end) as \'613 Health\',\n  sum(case when (dewey.dewey = 616) then 1 else 0 end) as \'616 Diseases\',\n  sum(case when (dewey.dewey = 796) then 1 else 0 end) as \'796 Sports\',\n  sum(case when (dewey.dewey = 917) then 1 else 0 end) as \'917 North America\',\n  sum(case when (dewey.dewey = 5) then 1 else 0 end) as \'5 Computer Programming\'\n\nfrom\n  dewey, activity\nwhere\n  dewey.bib = activity.bib and\n  dewey.dewey in (811, 599, 746, 398, 613, 616, 796, 917, 5) and\n  year(activity.o) in (2006, 2012)\ngroup by year(activity.o);";

  
  // Prep canvas
  size(1024,768);
  colorMode(HSB, 360, 100, 100);
  
  // Load fonts
  mainFont = createFont("CorsivaHebrew", 23);
  categoryFont = createFont("CorsivaHebrew", 14);
  titleFont = createFont("CorsivaHebrew", 15);
  yearFont = createFont("CorsivaHebrew", 30);
  
  msql = new MySQL( this, "tango.mat.ucsb.edu", "spl1", "mat259", "V1sual1zat1on" ); //host, database, user, password
  if ( msql.connect() ) { 
    msql.query(q);
    int row = 0;
    while (msql.next ()) {
      int col = 1;
      //println(msql.getInt(col+2) + DL + msql.getInt(col+3) + DL + msql.getInt(col+4) + DL + msql.getInt(col+5) + DL + msql.getInt(col+6) + DL + msql.getInt(col+7) + DL + msql.getInt(col+8));
      for (col = 0; col < categoryCount; col++) {
        int checkoutCount = msql.getInt(col+2);
        maxCheckoutCount = max(maxCheckoutCount, checkoutCount);
        checkouts[row][col] = checkoutCount;
      }
      row += 1;
    }
  }
  else {
    println("MySQL Connection failed!");
  }
  println("maxCheckoutCount: " + maxCheckoutCount);
  String[] fontList = PFont.list();
  //println(fontList);
}


int saturation = 75;
int brightness = 85;

float outerCircleRadius = 290.0;
float innerCircleRadius = outerCircleRadius-24.0;
float categoryRadius = (outerCircleRadius+innerCircleRadius)/2.0001;


float webOpacity = 90;

void draw() {
  noLoop();
  noStroke();
  background(360);
  textAlign(CENTER, CENTER);
  
  // "Checkouts by Dewey Category" label
  
  fill(0, 0, 25);
  textFont(mainFont);
  text("Checkouts by Dewey Category", width/2.0, 40);
  
  // "2006, 2012" label
  
  textFont(yearFont);
  int spacing = 55;
  text("2006", width/2.0-spacing, 80);
  text("2012", width/2.0+spacing, 80);
  
  
  // Year balls
  int ballDiameter = 16;
  stroke(0, 0, 100);
  fill(#f32a7e, webOpacity);
  ellipse(width/2.0-14, 83, ballDiameter, ballDiameter);
  fill(#2ec5f3, 94);
  ellipse(width/2.0+14, 83, ballDiameter, ballDiameter);
  
  stroke(0, 0, 65);
  line(width/2.0-7, 83, width/2.0+7, 83);
  
  // Dewey category title labels
  
  textFont(categoryFont);
  
  
  int diagramOffsetY = 30;
  
  pushMatrix();
  translate(width/2.0, height/2.0+diagramOffsetY);
  float rot = 0;
  for (int i = 0; i < categoryCount; i++, rot = i*(360.0/(categoryCount))) {
    
    pushMatrix();
    rotate(radians(rot));
    
    // line
    
    smooth();
    stroke(0, 0, 85);
    line(0, innerCircleRadius, 0, -innerCircleRadius);
    
    
    translate(0, categoryRadius);
    if (i >= 3 && i <= 5) {
      rotate(radians(180));
    }
    
    noSmooth();
    noStroke();
    fill(0, 0, 45);
    text(categories[i], 0, 0);
    popMatrix();
  }
  //popMatrix();
  
  
  
  smooth();
  
  // Web
  
  
  //noFill();
  stroke(30, 0, 100);
  fill(#f32a7e, webOpacity);
  strokeWeight(1.0);

  //pushMatrix();
  //translate(width/2.0, height/2.0);
  
  float webMaxRadius = innerCircleRadius-10.0;
   
  
  for (int yearIndex = 0; yearIndex < yearCount; yearIndex++) {
    
    if (yearIndex == 1) {
      fill(#2ec5f3, 94);
    }
    
    beginShape(); 
    // Drawings
    vertex(0, checkouts[yearIndex][0]/maxCheckoutCount*webMaxRadius);
    // Food & Drink
    float r = checkouts[yearIndex][1]/maxCheckoutCount*webMaxRadius/sqrt2;
    vertex(-r,r); 
    // Textile Arts
    vertex(-checkouts[yearIndex][2]/maxCheckoutCount*webMaxRadius, 0); 
    // Folklore
    r = checkouts[yearIndex][3]/maxCheckoutCount*webMaxRadius/sqrt2;
    vertex(-r,-r); 
    // Health
    vertex(0, -checkouts[yearIndex][4]/maxCheckoutCount*webMaxRadius);
    // Disease
    r = checkouts[yearIndex][5]/maxCheckoutCount*webMaxRadius/sqrt2;
    vertex(r,-r);
    // Sports
    vertex(checkouts[yearIndex][6]/maxCheckoutCount*webMaxRadius, 0); 
    // Food & Drink
    r = checkouts[yearIndex][7]/maxCheckoutCount*webMaxRadius/sqrt2;
    vertex(r,r);
    vertex(0, checkouts[yearIndex][0]/maxCheckoutCount*webMaxRadius);
   
    endShape();
    
  }
  
  // Cirlces
  
  strokeWeight(1.0);
  noFill();
  stroke(0, 0, 85);
  ellipse(0, 0, outerCircleRadius*2.0, outerCircleRadius*2.0);
  ellipse(0, 0, innerCircleRadius*2.0, innerCircleRadius*2.0);
  
  //popMatrix();
    
  //exit();
}




